/*
@memo   
@name Prototype
@author .. , .. 
@version 1.0
*/

#include "pardbms.h"	// 
#include "omegacfg.h"	//  
#include "dbparams.h"	//   
#include "globals.h"	//   
#include "compiler.h"	//   
#include "lineplan.h"	//    
#include "parallel.h"	//   
#include "agentmgr.h"	//    
#include "engine.h"		//   
#include "dbmgr.h"		//    
#include "mpi.h"		//   
#include <stdlib.h>
#include <assert.h>
#include <stdio.h>
#include <time.h>


/*
@memo  
*/
static int initflag=FALSE;

/*
@memo  (,  ,  ,  ,  )
*/
static treenode_t * QUERY_TREE;

/*
@memo  
*/
static treenode_t * PARALLEL_AGENT;

/*
          .
      .
@memo  
@return SUCCESS       
@see dbms_DoneDBMS dbms_RunDBMS
*/
void dbms_InitDBMS(int argc, char * argv[])
{
	int i, res;

	if (initflag)
		return;
	initflag=TRUE;
	res=MPI_Init(&argc, &argv);
	assert(res==MPI_SUCCESS);
	res=MPI_Comm_size(MPI_COMM_WORLD, &NODES_IN_CLUSTER);
	assert(res==MPI_SUCCESS);
	res=MPI_Comm_rank(MPI_COMM_WORLD, &WHOAMI);
	assert(res==MPI_SUCCESS);

//    -- 
// 0		1							2
// omega	___R0	___R1
	for (i=0; i<argc-1; i++) { 
		TUPLES_IN_FRAGMENT[i]=atol(argv[i+1]);
		assert(TUPLES_IN_FRAGMENT[i]>=TUPLES_IN_FRAGMENT_MIN && TUPLES_IN_FRAGMENT[i]<=TUPLES_IN_FRAGMENT_MAX);
	}
//       TUPLES_IN_FRAGMENT_MIN.
	for (i=0; i<RELATIONS_IN_DATABASE-argc+1; i++) 
		TUPLES_IN_FRAGMENT[i+argc-1]=TUPLES_IN_FRAGMENT_MIN;

	db_InitDatabaseManager();
	cp_InitCompiler();
	lp_InitPlanGenerator();
	pl_InitParallelizer();
	am_InitAgentManager();
	en_InitEngine();
}

/*
      .
@memo  
@return SUCCESS       
@see dbms_InitDBMS dbms_DoneDBMS
*/
void dbms_RunDBMS()
{
	int res;
	time_t starttime, stoptime;

	assert(initflag==TRUE);

	db_OpenDatabase();
	QUERY_TREE=cp_CompileQuery(QUERY_FNAME);
	lp_GeneratePlan(QUERY_TREE);
	pl_ParallelizeQuery(QUERY_TREE);
	PARALLEL_AGENT=am_MakeAgent(QUERY_TREE);

	res=MPI_Barrier(MPI_COMM_WORLD);
	assert(res==MPI_SUCCESS);
	
	time(&starttime);
	en_ExecuteQuery(PARALLEL_AGENT);
	time(&stoptime);

	res=MPI_Barrier(MPI_COMM_WORLD);
	assert(res==MPI_SUCCESS);

	EXECTIME=difftime(stoptime, starttime);
}

/*
          .
@memo  
@return SUCCESS       
@see dbms_InitDBMS dbms_RunDBMS
*/
void dbms_DoneDBMS(void)
{
	int res;

	assert(initflag==TRUE);	
	db_CloseDatabase();
	res=MPI_Finalize();
	assert(res==MPI_SUCCESS);
}

